/*
 * Copyright (C) 2006, 2007 Masahito Henmi, All rights reserved.
 */
package strawberry.renderkit;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.component.UIComponent;
import javax.faces.component.EditableValueHolder;
import javax.faces.convert.*;
import javax.faces.el.ValueBinding;
import javax.faces.render.Renderer;
import java.io.IOException;
import java.util.Map;
import java.util.Date;
import java.util.Locale;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.logging.*;
/**
 *  N   DateInputPanel _[  06/07/14
 * 
 *   DateDatag^Cv   ^\tgŎgĂ̂ŁAC
 */
public class DateInputPulldownRenderer extends Renderer {

  // jdk1.4 logging
  protected final static Logger log = Logger.getLogger("strawberry.ren.DateInputPulldownRenderer");

  SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");


  protected void encodeRecursive(FacesContext context, UIComponent component) throws IOException {

    String family = component.getFamily();
    String rendererType = component.getRendererType();
    Renderer rend = context.getRenderKit().getRenderer(family, rendererType);

    if (rend == null) {
        log.warning("Renderer not found!!    '" + family + "', '" + rendererType);
    } else {
      rend.encodeBegin(context, component);
      rend.encodeChildren(context, component);
      rend.encodeEnd(context, component);
    }
  }

  //------------------------------------------
  
  public Object getConvertedValue(FacesContext context, UIComponent component, Object submittedValue) throws javax.faces.convert.ConverterException {

    Object wk = _getConvertedValue( context,  component,  submittedValue);
    if (log.isLoggable(Level.FINE)) {
      if (wk == null) {
        log.fine("getConvertedValue -->  " + wk);
      } else {
        log.fine("getConvertedValue -->  " + wk + ", " + wk.getClass().getName());
      }
    }
    return wk;
  }

  /** x DateData쐬ǍAjava.sql.Dateɕϊ */
  private Object _getConvertedValue(FacesContext context, UIComponent component, Object submittedValue) throws javax.faces.convert.ConverterException {

    java.sql.Date rc = null;
    DateData userData = (DateData) submittedValue;
    try {
      rc = userData.parseDate();
    } catch (Exception e) {
      if (log.isLoggable(Level.WARNING)) {
        log.log(Level.WARNING, "catch Exception ----------------------------", e);
      }
      throw new ConverterException(  new FacesMessage("t͂Ă܂")
                                     );
    }
    return rc;
  }

  public void decode(FacesContext context, UIComponent component) {

    EditableValueHolder uii = (EditableValueHolder) component;
    Map requestMap = context.getExternalContext().getRequestParameterMap(); // 2006.12.04

    Locale currentLocale = Locale.getDefault();
    DateData userData = (DateData) uii.getSubmittedValue();
    if ( userData == null ) {

      Object obj = uii.getValue();
      if (obj == null) {
        userData = new DateData();
      } else if (obj instanceof Date) {
        userData = new DateData((Date) obj, currentLocale, null, false);
      } else if (obj instanceof String) {
        userData = new DateData();
        userData.setDate((String) obj);
      } else {
        log.warning("obj = " + obj + ", class = " + obj.getClass().getName());
      }
    }

    String clientId = component.getClientId(context);
    log.fine("decode --  " + clientId);

    if (clientId != null) {
      String yy = (String) requestMap.get(clientId + "_yy");
      String mm = (String) requestMap.get(clientId + "_mm");
      String dd = (String) requestMap.get(clientId + "_dd");

      yy = (yy == null) ? "" : yy.trim();
      mm = (mm == null) ? "" : mm.trim();
      dd = (dd == null) ? "" : dd.trim();

      log.finer("!!!!!!!!!! yy = " + yy + ", mm = " + mm + ", dd = " + dd);

      userData.setDay(   dd);
      userData.setMonth( mm);
      userData.setYear(  yy);
      uii.setSubmittedValue( userData );
    }
  }

  public void encodeBegin(FacesContext context, UIComponent component) throws IOException {

    if (log.isLoggable(Level.FINER))  log.finer("Begin encoding component " + component.getId());

    String  style      = (String) component.getAttributes().get("style");
    String  styleClass = (String) component.getAttributes().get("styleClass");

    ResponseWriter writer = context.getResponseWriter();

    if (spanned(component)) {
      writer.startElement("span", component);

      if (styleClass != null) {
        writer.writeAttribute("class", styleClass, "styleClass");
      }
      if (style != null) {
        writer.writeAttribute("style", style, "style");
      }
    }
  }

  public void encodeChildren(FacesContext context, UIComponent component) throws IOException {
  }


  public void encodeEnd(FacesContext context, UIComponent component) throws IOException {

    if (context == null || component == null) throw new NullPointerException();


    ResponseWriter writer = context.getResponseWriter();

    String clientId = component.getClientId(context);
    if (clientId == null) throw new NullPointerException();

    String id = component.getId();
    EditableValueHolder uii = (EditableValueHolder) component;

    DateData dateData = null;

    Object obj = uii.getValue();
    if (obj != null) {
      if (obj instanceof DateData) {
        log.fine("********************* 150s DateData");
        dateData = (DateData) obj;
      } else if (obj instanceof String) {
        log.fine("********************* 153s String");
        dateData = new DateData();
        dateData.setDate((String) obj);
      } else if (obj instanceof java.sql.Timestamp) {
        log.fine("********************* 157s Timestamp");
        java.sql.Timestamp ts = (java.sql.Timestamp) obj;
        String sdate = simpleDateFormat.format(ts);
        dateData = new DateData();
        dateData.setDate(sdate);
      } else if (obj instanceof java.sql.Date) {
        log.fine("********************* 163s java.sql.Date");
        java.sql.Date ts = (java.sql.Date) obj;
        String sdate = simpleDateFormat.format(ts);
        dateData = new DateData();
        dateData.setDate(sdate);
      } else {
        log.fine("********************* 169s " + obj.getClass().getName());
      }
    }

    if (dateData == null) obj = uii.getSubmittedValue();
    if (obj instanceof DateData) {
      dateData = (DateData) obj;
    } else if (obj instanceof String) {
      dateData = new DateData();
      dateData.setDate((String) obj);
    }

    log.fine("********************* 192s " + dateData);

    if (null == dateData) {

      Map initParam = context.getExternalContext().getInitParameterMap();
      
      GregorianCalendar cal3 = null;
      Object debug = initParam.get("debug.GregorianCalendar");
      if (debug != null) {
        cal3 = (GregorianCalendar) debug;
      } else {
        cal3 = new GregorianCalendar();
      }
      
      String strDate = new SimpleDateFormat("yyyy-MM-dd").format(cal3.getTime());

      dateData = new DateData();
      dateData.setDate(strDate);
    }
    
  //  ColumnCustomizerImpl cc = new ColumnCustomizerImpl();
  //  cc.setPreferredWidth(4);
    Object cc = null;

    String  strUnselectValue = (String) component.getAttributes().get("writeUnselectValue");

    // dwarf̌݊lāAftHg true
    boolean writeUnselectValue = true;
    if (strUnselectValue != null) {
      if ("false".equalsIgnoreCase(strUnselectValue)) {
        writeUnselectValue = false;
      }
    }

    int tabIndex = TABINDEX_HAS_NO_VALUE;

    javax.faces.component.html.HtmlInputText htmlInputText = null;
    
    if (component instanceof javax.faces.component.html.HtmlInputText) {
      htmlInputText = (javax.faces.component.html.HtmlInputText) component;
    } else {
      log.fine("(213) component = " +  component.getClass().getName() );
    }
    
    if (htmlInputText != null) {
      String strTab = htmlInputText.getTabindex();
      log.info("(220) strTab = " + strTab);
      
      if (strTab != null) {
        tabIndex = Integer.parseInt(strTab);
      }
    }

  //  RendererConfig rConf = getRendererConfig();
    int fromYear = 2005; //rConf.getInt(context, Constants.RENDER_PULLDOWN_FROM_YEAR);
    Object  wkYearFrom = component.getAttributes().get("yearFrom");
    if (wkYearFrom == null) {
      log.fine("(226) yearFrom = null");
    } else if (wkYearFrom instanceof String) {
      String strYearFrom = (String) wkYearFrom;
      fromYear = Integer.parseInt(strYearFrom);
    } else if (wkYearFrom instanceof Long) {
      Long lngYearFrom = (Long) wkYearFrom;
      fromYear = lngYearFrom.intValue();
    } else {
      log.fine("(231) class = " +  wkYearFrom.getClass().getName() );
    }
    if (fromYear == 0) fromYear = 2000;
    
    int toYear = 2010; //rConf.getInt(context, Constants.RENDER_PULLDOWN_TO_YEAR);
    Object  wkYearTo = component.getAttributes().get("yearTo");
    if (wkYearTo == null) {
      log.fine("(228) yearTo = null");
    } else if (wkYearTo instanceof String) {
      String strYearTo = (String) wkYearTo;
      toYear = Integer.parseInt(strYearTo);
    } else if (wkYearTo instanceof Long) {
      Long lngYearTo = (Long) wkYearTo;
      toYear = lngYearTo.intValue();
    } else {
      log.fine("(234) class = " +  wkYearTo.getClass().getName() );
    }
    if (toYear == 0) toYear = 2010;

    
    writeInputTag( writer, clientId + "_yy", id + "_yy",
                  dateData.getYear(), context, cc, writeUnselectValue,
                   fromYear, toYear, tabIndex);
    writer.writeText("N","N");
    if (tabIndex >= 0) tabIndex++;

    writeInputTag( writer, clientId + "_mm", id + "_mm",
                  dateData.getMonth(), context, cc, writeUnselectValue,
                   1, 12, tabIndex);
    writer.writeText("","");
    if (tabIndex >= 0) tabIndex++;

    writeInputTag( writer, clientId + "_dd", id + "_dd",
                  dateData.getDay(), context, cc, writeUnselectValue,
                   1, 31, tabIndex);
    writer.writeText("","");

    if (spanned(component)) writer.endElement("span");
    if (log.isLoggable(Level.FINER))   log.finer("End encoding component " +  component.getId());
  }

  static final int TABINDEX_HAS_NO_VALUE = -9999;

  
  private void writeInputTag(ResponseWriter writer, String clientId, String id, String value,
                             FacesContext context,
                             /* ColumnCustomizerImpl*/  Object   cc,
                             boolean writeUnselectValue,
                             int from, int to, int tabIndex) throws IOException {

    writer.startElement("select", null );
    writer.writeAttribute("name", clientId, "clientId");
    if (tabIndex != TABINDEX_HAS_NO_VALUE) {
      writer.writeAttribute("tabindex", "" + tabIndex, "tabindex");
    }

    writePulldown(writer, from, to, value, writeUnselectValue);

    writer.endElement("select");
  }

  private void writePulldown(ResponseWriter writer, int start, int end,
                             String value, boolean writeUnselectValue) throws IOException {

    if (writeUnselectValue) {
      writer.startElement("option", null);
      writer.writeAttribute("value", "unselected", "value");
      writer.writeText("@", null);
      writer.endElement("option");
    }

    
    for (int i = start; i <= end; i++) {
      writer.startElement("option", null);
      String str = "" + i;
      writer.writeAttribute("value", str, "value");

      //IĂlselectedɂ
      if (value != null) {
        if (compareString(value, str)) writer.writeAttribute("selected", "selected", "selected");
      }

      writer.writeText(str, null);
      writer.endElement("option");
    }
  }

  /**
   *  擪 0ꍇAĔrB<br>
   *  caoV̎, 07ƓĂāAL̃[vō쐬Ă錎
   *  7ƂȂAvȂߍ쐬
   */
  public boolean compareString(String val1, String val2) {
    if (val1.startsWith("0")) {
      val1 = val1.substring(1);
    }
    if (val2.startsWith("0")) {
      val2 = val1.substring(1);
    }
    return val1.equals(val2);
  }


  private boolean spanned(UIComponent component) {
    return true;
  }
}
